# AUI

[![React](https://img.shields.io/badge/react-^15.6.1-brightgreen.svg?style=flat-square)](https://github.com/facebook/react)
[![npm version](https://img.shields.io/npm/v/redux.svg?style=flat-square)](https://www.npmjs.com/package/redux)
[![roadhog](https://img.shields.io/badge/roadhog-1.2.2-blue.svg?style=flat-square)](https://github.com/sorrycc/roadhog)
[![MIT](https://img.shields.io/dub/l/vibe-d.svg?style=flat-square)](http://opensource.org/licenses/MIT)
[![js-standard-style](https://img.shields.io/badge/code%20style-standard-brightgreen.svg)](http://standardjs.com)

演示地址：(待补充)

## 定位：
-   web页纯组件库，不涉及三方逻辑和太多业务场景代码。

## 组件规范：
为了减少编码过程中可能出现的问题，我们制定了如下规范，希望能减少常见的错误。
-   有状态组件只有 render 方法才能返回 JSX，因为 JSX 中的虚拟 DOM 有一个 _owner 属性，这与它与组件实例进行绑定。如果其他方法里使用了 JSX， _owner 就没有与组件实例进行绑定。
-   render 方法里面应该以 < 开头，不应该存在 if else 分支，视情况返回不同的JSX。相同的组件应该返回相同的顶级元素容器。
```
// 不好的实践
render() {
   if (this.state.a) {
      return <strong>不要</strong>
   } else {
      return <div>这样</div>
   }
}
```
-   ref 应该尽快淘汰字符串形式，因为字符串形式的 ref 会自始至终将字符串放在 refs 对象中，会有泄露的问题。
```
// 不好的实践
<Foo
  ref="myRef"
/>

// 一般的实践
<Foo
  ref={(ref) => { this.myRef = ref; }}
/>
```
-   上面的方法之所以是一般的实践，而不是好的实践，是因为我们在查看组件时，别人很难察觉到你在 JSX 里偷偷为组件添加了一个新属性。组件所有用到的属性，应该都能在 constructor 或 defaultProps 中找到。

-   refs.xxx 中的 DOM 节点，不应该再转存到组件实例上或其他地方中。每次访问 refs.xxx 必须判定其是否为空。

-   不要在 componentWillUpdate/componentDidUpdate/render 中执行 setState, 可能导致死循环。

-   不要在 JSX 中使用 bind 方法绑定组件实例
```
// 不好的实践
class extends React.Component {
  onClickDiv() {
    // do something
  }

  render() {
    return <div onClick={this.onClickDiv.bind(this)} />;
  }
}

// 好的实践
class extends React.Component {
  constructor(props) {
    super(props);

    this.onClickDiv = this.onClickDiv.bind(this);
  }

  onClickDiv() {
    // do something
  }

  render() {
    return <div onClick={this.onClickDiv} />;
  }
}
```
-   不要使用 cloneElement，createElement，让 JSX 与 babel 帮你创建它们。

-   不要使用 createClass, mixin，PropTypes（它们已经被移出核心库，被逐渐边缘化，有关属性的描述改成文档注释吧）。

## 更新日志

### 0.0.1

`2017-07-18`

-初步搭建脚手架

### 0.0.2

`2017-09-22`

-业务封装

## 开发构建

### 目录结构

```bash

├── /assets/         # 项目截图或PRD图
├── /dist/           # 项目输出目录Build生产文件
├── /node_modules/   # 模块依赖资源包
├── /src/            # 项目源码目录
│ ├── /components/   # 顶层全局UI组件及UI相关方法
│ ├── /styles/       # 项目所有样式
│ ├── /utils/        # 工具函数
│ ├── index.js       # 入口文件
├── package.json     # 项目信息
├── .eslintrc        # Eslint配置
└── .roadhogrc.js    # roadhog配置
```

文件夹命名说明:

-   components：组件（方法）为单位以文件夹保存，文件夹名组件首字母大写（如`DataTable`），方法首字母小写（如`layer`）,文件夹内主文件与文件夹同名，多文件以`index.js`导出对象（如`./src/components/Layout`）。
-   routes：页面为单位以文件夹保存，文件夹名首字母小写（特殊除外，如`UIElement`）,文件夹内主文件以`index.js`导出。

### 开发

克隆项目文件:

    git clone git@github.com:EigenLab/AUI.git

进入目录安装依赖:

    npm install(npm i) 或者 yarn install

构建：

```bash
npm run build

将会生成dist目录
```


使用：

```bash
import A from 'agui'
```